from ._enums import MILLISECONDS as MILLISECONDS, SECONDS as SECONDS, SocketEvt as SocketEvt, Time as Time
from ._exceptions import CurlError as CurlError
from .abc import RequestProtocol as RequestProtocol
from .scalars import Quantity as Quantity
from socket import socket as Socket
from typing import Self, TypeAlias, TypeVar

T = TypeVar('T')
Event: TypeAlias

class Multi:
    def __init__(self) -> None: ...
    async def process(self, request: RequestProtocol[T]) -> T: ...

class _ExternalSocket(Socket):
    def __del__(self) -> None: ...
    @classmethod
    def from_fd(cls, fd: int) -> Self: ...



from pycurl import Curl as Curl
from typing import Protocol, TypeVar

T_co = TypeVar('T_co', covariant=True)

class RequestProtocol(Protocol[T_co]):
    def configure_handle(self, handle: Curl) -> None: ...
    def has_response(self) -> bool: ...
    def response(self) -> T_co: ...
    def completed(self) -> T_co: ...



import enum
from typing import Generic, Self, TypeVar, overload

U = TypeVar('U', bound='QuantityUnit')

class Quantity(int, Generic[U]):
    @overload
    def __add__(self, other: Quantity[U]) -> Quantity[U]: ...
    @overload
    def __add__(self, other: int) -> int: ...
    @overload
    def __sub__(self, other: Quantity[U]) -> Quantity[U]: ...
    @overload
    def __sub__(self, other: int) -> int: ...
    def __mul__(self, other: int | float) -> Quantity[U]: ...
    def __truediv__(self, other: int | float) -> Quantity[U]: ...

class QuantityUnit(enum.Enum):
    @property
    def value(self) -> int: ...
    def __rmatmul__(self, scalar: float | int) -> Quantity[Self]: ...
    def __rrshift__(self, quantity: Quantity[Self]) -> float: ...
    def __rfloordiv__(self, quantity: Quantity[Self]) -> int: ...



import pycurl
from typing import IO

class Request:
    url: str
    destination: IO[bytes]
    def __init__(self, url: str, destination: IO[bytes]) -> None: ...
    def configure_handle(self, handle: pycurl.Curl) -> None: ...
    def has_response(self) -> bool: ...
    def response(self) -> None: ...
    def completed(self) -> None: ...
